/**
  *  \file client/cscreen.h
  *  \brief Control Screens
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_CSCREEN_H
#define PCC_V2_CLIENT_CSCREEN_H

#include "client/chart.h"
#include "game/objtypes.h"
#include "ui/commands.h"
#include "ui/ui.h"
#include "cpluslib/holder.h"
#include "client/wtile.h"
#include "client/screens.h"
#include "client/widgets/scanresult.h"
#include "int/keymap.h"

class WControlScreen;
class WScannerChartWidget;

class WControlScreen : public UIFixedWidget {
 protected:
    ObjectHolder holder;

 private:
    class Skin;

    /* Identification */
    WScreenId::ScreenId  screen_id;            ///< Screen Id (function key number).
    IntKeymapRef         keymap;               ///< Keymap.
    GObjectType&         type;                 ///< Object type.
    SignalConnection     conn_pos_changed;     ///< Connection to type's sig_pos_changed.

    /* Widgets */
    WScanResult          scanner;              ///< Scanner widget.
    WScannerChartWidget* chart;                ///< Starchart widget.

    /* Tile positioning */
    int                  tile_width;           ///< Width of one column.
    int                  tile_y;               ///< Current Y.
    int                  tile_x;               ///< Current X.
    int                  tile_new_y;           ///< Y for new columns.
    int                  chart_y2;             ///< Bottom Y of starchart widget. Top Y is derived from our top.
    int                  chart_x1;             ///< Left X of starchart widget. Right X is derived from our right X.

    /* Control */
    WScreenId            return_value;         ///< Return value.
    static WControlScreen* current_cscreen;    ///< Pointer to current control screen, if any.

    void setGeometry();
    void onPosChanged(int);

 protected:
    void addChart(WScannerChartWidget& chart);
    void addTile(WTile& tile);
    void addTile(WTile& tile, const char* title);
    void setKeymap(IntKeymapRef keymap);

 public:
    WControlScreen(GObjectType& atype, WScreenId::ScreenId screen_id);
    ~WControlScreen();

    void          drawContent(GfxCanvas& can);
    bool          handleEvent(const UIEvent& e, bool second_pass);

    void          setCurrent(int id);
    void          doBrowse(bool forward, bool marked);

    GObjectType&  getType();
    GPoint        getScannerPosition() const;
    WScanResult&  getScanner();
    WScreenId::ScreenId getScreenId() const;

    virtual void  update() = 0;
    virtual void  setScriptContext() = 0;

    WScreenId     runScreen();
    void          stopWithScreenId(WScreenId id);

    static WControlScreen* getCurrentControlScreen();
};

/** Get contained type. */
inline GObjectType&
WControlScreen::getType()
{
    return type;
}

/** Get contained scanner view. */
inline WScanResult&
WControlScreen::getScanner()
{
    return scanner;
}

/** Get screen Id. */
inline WScreenId::ScreenId
WControlScreen::getScreenId() const
{
    return screen_id;
}

/** Get active control screen Id.
    \return control screen object; 0 if none active */
inline WControlScreen*
WControlScreen::getCurrentControlScreen()
{
    return current_cscreen;
}

#endif
