/**
  *  \file client/tiles/planetcolonist.cc
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/planetcolonist.h"
#include "ui/widgets.h"
#include "util/nls.h"
#include "cpluslib/format.h"
#include "game/planet.h"
#include "util/misc.h"
#include "game/tables.h"

/************************** WPlanetColonistTile **************************/

/** \class WPlanetColonistTile
    \brief Planet Colonists Tile

    Displays information about the planetary colony.

    <pre>
    Population:    XX
    Tax Rate:      XX%
    Colonists are XYZ.
    </pre> */

WPlanetColonistTile::WPlanetColonistTile(GObjectType& type)
    : WObjectTypeObserverTile(GfxPoint(315, 3 * font_heights[FONT_NORMAL]), type)
{
    setLayout(RightJustified, 0, true);
    addControl(GfxRect(0, 3 * font_heights[FONT_NORMAL] - 20, 20, 20), h.add(new UIButton("T", 't')));
}

void
WPlanetColonistTile::drawPlanetColonistTile(UIAbstractTableWriter& w, bool /*all_cols*/)
{
    w.newLColumn();
    w.writeFixedCell(UIColor::tc_Static, _("Population:"));
    w.writeFixedCell(UIColor::tc_Static, _("Tax Rate:"));
    w.writeFixedCell(UIColor::tc_Static, _("Colonists are"));

    if (GPlanet* p = dynamic_cast<GPlanet*>(getObject())) {
        w.newLColumn(200);
        w.writeVariableCell(UIColor::tc_Green, clansToString(p->getColonists()));
        w.writeVariableCell(UIColor::tc_Green, format("%d%%", p->getColonistTax()));

        int happy = p->getColonistHappiness();
        UISkin::Color c = (happy <= 40
                           ? UIColor::tc_Red
                           : happy < 70
                           ? UIColor::tc_Yellow
                           : UIColor::tc_Green);
        w.writeVariableCell(c, format("%s (%d)", getHappinessName(happy), happy));
    }

    w.finish();
}

void
WPlanetColonistTile::drawData(GfxCanvas& can)
{
    UITableWriter tw(can, getSkin(), getInnerSize(5, 20), FONT_NORMAL);
    drawPlanetColonistTile(tw, true);
}

