/**
  *  \file client/tiles/planetfcode.cc
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/planetfcode.h"
#include "ui/widgets.h"
#include "int/intglobal.h"
#include "int/value.h"
#include "int/data.h"
#include "game/planet.h"
#include "util/nls.h"
#include "cpluslib/format.h"

/**************************** WPlanetFCodeTile ***************************/

/** \class WPlanetFCodeTile
    \brief Planet FCode & Money Tile

    Displays the planet's FCode and cash. */

WPlanetFCodeTile::WPlanetFCodeTile(GObjectType& type)
    : WObjectTypeObserverTile(GfxPoint(315, 4*font_heights[FONT_NORMAL]), type)
{
    setLayout(RightJustified, 0, false);
    addControl(GfxRect(10, 0,                              20, 20), h.add(new UIButton("F", 'f')));
    addControl(GfxRect(0,  4*font_heights[FONT_NORMAL]-20, 30, 20), h.add(new UIButton("F9", SDLK_F9)));
}

void
WPlanetFCodeTile::drawData(GfxCanvas& can)
{
    const int y = font_heights[FONT_NORMAL];

    if (GPlanet* p = dynamic_cast<GPlanet*>(getObject())) {
        setColor(UIColor::tc_Static);
        outText(can, FONT_NORMAL, 5, 0, _("Funds:"));
        outText(can, FONT_NORMAL, 5, y, _("Friendly Code:"));

        setColor(UIColor::tc_Green);
        outTextF(can, FONT_NORMAL, 65, 0, 170, format("%d mc", p->getCargo(el_Money)));
        outTextF(can, FONT_NORMAL, 125, y, 70, p->getFCode());

        string_t comment;
        if (IntValue* v = getPlanetProperties(p->getId())[pp_Comment])
            comment = v->toString(false);
        if (comment.length()) {
            setColor(UIColor::tc_Yellow);
            outTextF(can, FONT_NORMAL, 5, 5*y/2, extent.w-35, comment);
        } else {
            setColor(UIColor::tc_Static);
            outTextF(can, FONT_NORMAL, 5, 5*y/2, extent.w-35, _("(press [F9] to add a note)"));
        }
    }
}

