/**
  *  \file flak/pdk-play.cc
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <cstring>
#include <phostpdk.h>
#include "io/streams.h"
#include "util/except.h"
#include "flak/flak-db.h"
#include "util/console.h"
#include "util/modinit.h"

static const char* filename;
static const char* progname;

static void
help()
{
    ErrorExit("usage: %s FILE [GAMEDIR [ROOTDIR]]", progname);
}

static void
play(FlakVcrEntry& e)
{
    FlakBattle& b = e.getBattle();
    FlakNullVisualizer vis(b);
    Info("  Time according to header: %7d", e.getTotalTime());
    b.init();
    while (!b.playOneTick())
        ;
    Info("  Real time taken:          %7d", b.getTime());
}

int
main(int /*argc*/, char** argv)
{
    bool had_gamedir = false, had_rootdir = false;
    progname = argv[0];
    while (const char* p = std::strpbrk(progname, ":/\\"))
        progname = p+1;
    while (*++argv) {
        if (filename == 0)
            filename = *argv;
        else if (!had_gamedir)
            gGameDirectory = *argv, had_gamedir = true;
        else if (!had_rootdir)
            gRootDirectory = *argv, had_rootdir = true;
        else
            help();
    }
    if (!filename)
        help();

    performInitialisation(OnStartup, mod_base_console);
    InitPHOSTLib();
    if (!ReadGlobalData())
        ErrorExit("Unable to read global data");
//     if (!Read_HConfig_File())
//         ErrorExit("Unable to read host config");
    if (!ReadHostData())
        ErrorExit("Unable to read host data");

    try {
        /* Now read the input file */
        Ptr<Stream> io = Stream::openFileNT(filename, Stream::C_READ);
        if (!io)
            ErrorExit("Unable to open file %s", filename);

        GFlakVcrDatabase db(*io);

        /* Play it */
        for (int i = 0; i < db.getCount(); ++i) {
            Info("Battle %d...", i+1);
            play(db.getBattle(i));
        }
    }
    catch (std::exception& e) {
        ErrorExit("Exception: %s", e.what());
    }
    catch (...) {
        ErrorExit("Unknown exception");
    }
    return 0;
}
