/**
  *  \file game/coord.h
  *  \brief Game Coordinate System Handling
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_COORD_H
#define PCC_V2_GAME_COORD_H

#include <iosfwd>
#include "cpluslib/string.h"

//! Point in Game Universe.
struct GPoint {
    int  x, y;
    GPoint(int ax, int ay)
        : x(ax), y(ay) { }
    GPoint() { }

    bool   isOnMap() const;
    GPoint getNormalized() const;
    GPoint getNearestTo(GPoint a) const;

    bool    operator==(GPoint rhs)   const { return x == rhs.x && y == rhs.y; }
    bool    operator!=(GPoint rhs)   const { return x != rhs.x || y != rhs.y; }
    GPoint  operator+ (GPoint other) const { return GPoint(x + other.x, y + other.y); }
    GPoint  operator- (GPoint other) const { return GPoint(x - other.x, y - other.y); }
    GPoint& operator+=(GPoint other)       { x += other.x; y += other.y; return *this; }
    GPoint& operator-=(GPoint other)       { x -= other.x; y -= other.y; return *this; }

    bool parseCoordinates(const string_t& s);
    bool parseSectorNumber(const string_t& s);
    bool parseSectorNumber(int n);

    int  getSectorNumber() const;
};

//! Configuration.
struct GChartConfiguration {
    /* public writable members */
    bool wrapped;               ///< true iff Sphere is used.
    int  xmin, ymin;            ///< minimum coordinates.
    int  xmax, ymax;            ///< maximum coordinates.

    /* derived stuff */
    void init();    
    int  xsize, ysize;          ///< size.

    /* Map images */
    int    getNumMapImages() const;
    GPoint getPointAlias(GPoint pt, int nr) const;
};

extern GChartConfiguration chart_conf;

long distanceSquaredRaw(GPoint a, GPoint b);
long distanceSquared(GPoint a, GPoint b);

void initCoordinates();

std::ostream& operator<<(std::ostream&, GPoint pt);

#endif
