/**
  *  \file int/unary.h
  *  \brief Interpreter: Unary Operations
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_UNARY_H
#define PCC_V2_INT_UNARY_H

#include "cpluslib/types.h"

class IntValue;

/** Minor opcode for unary operations. These are stored in the "minor" field of an
    IntOpcode whose major opcode is IntOpcode::maUnary. */
enum IntUnaryOperation {
    unNot,                      ///< Logical Not.
    unBool,                     ///< Cast to bool (double Not).
    unNeg,                      ///< Arithmetical negation.
    unPos,                      ///< Arithmetical non-negation (double Neg, Plus).
    unSin,                      ///< Sine.
    unCos,                      ///< Cosine.
    unTan,                      ///< Tangent.
    unZap,                      ///< "Zap" operation ('Z(n)').
    unThrow,                    ///< Throw argument.
    unAbs,                      ///< Absolute value.
    unExp,                      ///< Exponential function.
    unLog,                      ///< Natural logarithm.
    unBitNot,                   ///< Bitwise not.
    unIsEmpty,                  ///< Check for emptiness.
    unIsNum,                    ///< Check for numericness.
    unIsString,                 ///< Check for string.
    unAsc,                      ///< Get ASCII code of stringified arg.
    unChr,                      ///< Make string from ASCII code.
    unStr,                      ///< Convert to string.
    unSqrt,                     ///< Square root.
    unTrunc,                    ///< Truncate to integer.
    unRound,                    ///< Round to integer.
    unLTrim,                    ///< Trim string left.
    unRTrim,                    ///< Trim string right.
    unLRTrim,                   ///< Trim string left and right.
    unLength,                   ///< String length.
    unVal,                      ///< Convert string to number.
    unTrace,                    ///< Generate trace message.
    unNot2,                     ///< Logical Not for two-valued logic, empty counts as false.
    unAtom,                     ///< Convert string to integer.
    unAtomStr,                  ///< Convert previously converted integer back into string.
    unKeyCreate,                ///< Create named keymap.
    unKeyLookup,                ///< Look up named keymap.
    unInc,                      ///< Increment numeric.
    unDec                       ///< Decrement numeric.
};

const char* getUnaryName(uint8_t op);

IntValue* executeUnaryOperation(uint8_t op, IntValue* arg);

#endif
