/**
  *  \file confpars.h
  *  \brief Configuration Parser
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CONFPARS_H
#define PCC_V2_CONFPARS_H

#include <string>
#include <set>
#include "cpluslib/string.h"
#include "util/filepars.h"

class Stream;
class Config;
class ConfigOption;

/* confpars.cc */

class ConfigParser : public FileParser {
    string_t section;
    bool     in_section;
 public:
    ConfigParser();

    /** Assign value. Descendants must override this.
        \param line_id line number
        \param name    option name
        \param value   option value
        \param line    complete line */
    virtual void assign(int line_id, string_t name, string_t value, string_t line) = 0;
    virtual void error(int line_id, string_t msg);

    void configure(string_t section, bool in_section);

    bool isInSection() const;
    string_t getSectionName() const;

    void process(int line_id, string_t line);
};

/* conf-file.cc */

class ConfigFileParser : public ConfigParser {
    Config& the_config;
    bool ignore_unknown;

 public:
    ConfigFileParser(Config& the_config, bool ignore_unknown);
    virtual void assign(int line_id, string_t name, string_t value, string_t line);
    virtual void error(int line_id, string_t msg);
};

class ConfigFileUpdater : public ConfigParser {
    std::set<ConfigOption*> written_options;
    Config& the_config;
    Stream& out;

 public:
    ConfigFileUpdater(Config& the_config, Stream& out);
    virtual void assign(int line_id, string_t name, string_t value, string_t line);
    virtual void error(int line_id, string_t msg);
    virtual void unparsed(int line_id, string_t line);

    void finish();
};

#endif
